﻿using CallCenter.Caches;
using CallCenter.Devices;
using CallCenter.Notifications;
using CallCenter.Realtimes;
using CallCenter.Repository.SqlSugar;
using CallCenter.Share.Enums;
using CallCenter.Tels;
using MediatR;
using Microsoft.Extensions.Options;
using NewRock.Sdk;
using NewRock.Sdk.Control.Request;
using NewRock.Sdk.Control.Response;
using XF.Domain.Cache;
using XF.Domain.Exceptions;

namespace CallCenter.Application.Handlers
{
    public class IdleNotificationHandler : INotificationHandler<IdleNotification>
    {
        private readonly ITelCacheManager _telCacheManager;
        private readonly ITypedCache<Tel> _typedCache;
        private readonly IUserCacheManager _userCacheManager;
        private readonly IRealtimeService _realtimeService;
        private readonly IDeviceManager _deviceManager;
        private readonly ITelRestRepository _telRestRepository;


        public IdleNotificationHandler(ITelCacheManager telCacheManager, ITypedCache<Tel> typedCache, IUserCacheManager userCacheManager, IRealtimeService realtimeService, IDeviceManager deviceManager, ITelRestRepository telRestRepository)
        {
            _telCacheManager = telCacheManager;
            _typedCache = typedCache;
            _userCacheManager = userCacheManager;
            _realtimeService = realtimeService;
            _deviceManager = deviceManager;
            _telRestRepository = telRestRepository;
        }

        public async Task Handle(IdleNotification notification, CancellationToken cancellationToken)
        {
            var telModel = _telCacheManager.GetTel(notification.TelNo);
            telModel.TelStatus = Tels.ETelStatus.Ready;
            _typedCache.Set(notification.TelNo, telModel);

            var iswork = await _userCacheManager.IsWorkingByTelAsync(notification.TelNo, cancellationToken);
            if (!iswork)
                throw new UserFriendlyException(notification.TelNo + "未查询到工作记录");
            var restingTel = await _telRestRepository.GetAsync(d => d.TelId == telModel.Id && !d.EndTime.HasValue, cancellationToken);
            if (restingTel !=null)
                throw new UserFriendlyException("未查询到分机休息信息");

            foreach (var group in telModel.Groups)
            {
                await _deviceManager.AssginConfigGroupAsync(group.No, group.Distribution, new List<string>() { notification.TelNo },group.Voice,cancellationToken);
            }


            var workModel = _userCacheManager.GetWorkByTel(notification.TelNo);
            if (workModel != null)
            {
                await _realtimeService.IdleAsync(workModel.UserId, cancellationToken);
            }
        }
    }
}
