﻿using CallCenter.Caches;
using CallCenter.Notifications;
using CallCenter.Realtimes;
using CallCenter.Share.Enums;
using CallCenter.Share.Notifications;
using MediatR;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CallCenter.Application.Handlers.FlowControl
{
    public class AnswerExtToExtNotificationHandler : INotificationHandler<AnswerExtToExtNotification>
    {
        private readonly IRealtimeService _realtimeService;
        private readonly IUserCacheManager _userCacheManager;

        public AnswerExtToExtNotificationHandler(IRealtimeService realtimeService,IUserCacheManager userCacheManager)
        {
            _realtimeService = realtimeService;
            _userCacheManager = userCacheManager;
        }

        public async Task Handle(AnswerExtToExtNotification notification, CancellationToken cancellationToken)
        {
            //通知前端主叫
            //获取主叫工作信息
            var fromWork = _userCacheManager.GetWorkByTel(notification.FromTelNo);
            if (fromWork != null)
            {
                await _realtimeService.AnsweredAsync(fromWork.UserId, new AnseredDto() { Id = notification.FromTelNo,From= notification.FromTelNo,To = notification.ToTelNo , CallType= ECallType.ExtToExt }, cancellationToken);
            }
            //通知前端被叫
            var toWork = _userCacheManager.GetWorkByTel(notification.ToTelNo);
            if (toWork != null)
            {
                await _realtimeService.AnsweredAsync(toWork.UserId, new AnseredDto() { Id = notification.ToTelNo,From = notification.FromTelNo,To = notification.ToTelNo, CallType = ECallType.ExtToExt }, cancellationToken);
            }
        }
    }
}
